// $Id: alsa_mixer.cc,v 1.15 2003/03/11 19:40:42 gordon Exp $	
//
// Copyright (C) 2002 Gordon Fraser <gordon@debian.org>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// See the COPYING file for more details
//

#include "alsa_mixer.h"


//----------------------------------------------------------------------
AlsaMixer::AlsaMixer()
  throw(MixerDeviceException)
  : Mixer("default"), changed_state_(true)
{
  try
  {
    initMixer();
  }
  catch(MixerDeviceException& e)
  {
    std::cerr << "Opening Mixer failed." << std::endl;
    throw;
  }
}

//----------------------------------------------------------------------
AlsaMixer::AlsaMixer(const string& name)
  throw(MixerDeviceException)
  : Mixer(name), changed_state_(true)
{
  std::cerr << "Opening Mixer" << std::endl;
  
  try
  {
    initMixer();
  }
  catch(MixerDeviceException& e)
  {
    std::cerr << "Opening Mixer " << name << " failed." << std::endl;
    throw;
  }
}

//----------------------------------------------------------------------
AlsaMixer::~AlsaMixer()
{
  snd_mixer_free(mixer_handle_);
  snd_mixer_close(mixer_handle_);
}

//----------------------------------------------------------------------
void AlsaMixer::initMixer()
  throw(MixerDeviceException)
{
  int error;

  // Open Mixer

  if((error = snd_mixer_open(&mixer_handle_, 0) < 0))
  {
    throw MixerDeviceException(snd_strerror(error));
  }
  
  if((error = snd_mixer_attach(mixer_handle_, device_name_.c_str())) < 0)
  {
    throw MixerDeviceException(snd_strerror(error));
  }
  
  if((error = snd_mixer_selem_register (mixer_handle_, NULL, NULL)) < 0)
  {
    throw MixerDeviceException(snd_strerror(error));
  }
  
  //   snd_mixer_set_callback (mixer_handle_, &alsaMixerEvent);
  
  if((error = snd_mixer_load (mixer_handle_)) < 0)
  {
    throw MixerDeviceException(snd_strerror(error));
  }
  
  snd_mixer_elem_t *element;
  
  AlsaDevice* device = NULL;
  
  for (element = snd_mixer_first_elem(mixer_handle_); 
       element; 
       element = snd_mixer_elem_next(element))
  {
    if (!snd_mixer_selem_is_active(element))
      continue;

    device = NULL;      
    
    try
    {
      device = new AlsaDevice(element, snd_mixer_selem_get_name(element));
    }
    catch(InvalidChannelException& e)
    {
      std::cerr << e.getErrorMessage() << std::endl;
      delete device;
      device = NULL;
      continue;
    }
    
    if(device != NULL && device->isSupported())
    {
      devices_.push_back(device);
      num_devices_++;
    }
    else
    {
      delete device;
    }

  }
}



//----------------------------------------------------------------------
void AlsaMixer::muteChannel(unsigned channel_num)
  throw(InvalidChannelException)
{
  if(!isValidChannel(channel_num))
    throw InvalidChannelException(channel_num);

  devices_[channel_num]->mute();
}

//----------------------------------------------------------------------
void AlsaMixer::unMuteChannel(unsigned channel_num)
  throw(InvalidChannelException)
{
  if(!isValidChannel(channel_num))
    throw InvalidChannelException(channel_num);

  devices_[channel_num]->unMute();
}

//----------------------------------------------------------------------
void AlsaMixer::setChannelLeft(unsigned channel_num, unsigned value)
  throw(InvalidChannelException)
{
  if(!isValidChannel(channel_num))
    throw InvalidChannelException(channel_num);
  
  devices_[channel_num]->setLeftValue(value);
}

//----------------------------------------------------------------------
void AlsaMixer::setChannelRight(unsigned channel_num, unsigned value)
  throw(InvalidChannelException)
{
  if(!isValidChannel(channel_num))
    throw InvalidChannelException(channel_num);

  devices_[channel_num]->setRightValue(value);
}

//----------------------------------------------------------------------
bool AlsaMixer::toggleChannelRecord(unsigned channel_num)
  throw(InvalidChannelException)
{
  if(!isValidChannel(channel_num))
    throw InvalidChannelException(channel_num);

  return devices_[channel_num]->toggleCapture();
}

//----------------------------------------------------------------------
bool AlsaMixer::hasChanged()
{
  //  return changed_state_;
  return true;
}

//----------------------------------------------------------------------
bool AlsaMixer::getCapture(unsigned channel_num) const
  throw(InvalidChannelException)
{
  if(!isValidChannel(channel_num))
    throw InvalidChannelException(channel_num);

  return devices_[channel_num]->getCapture();
}

//----------------------------------------------------------------------
void AlsaMixer::doUpdate()
{
  snd_mixer_handle_events(mixer_handle_);
  
  for(unsigned count=0; count<num_devices_; count++)
  {
    devices_[count]->getLeftValue();
    devices_[count]->getRightValue();
  }
}

//----------------------------------------------------------------------
void AlsaMixer::doUpdate(unsigned channel_num)
  throw(InvalidChannelException)
{
  if(!isValidChannel(channel_num))
    throw InvalidChannelException(channel_num);

  snd_mixer_handle_events(mixer_handle_);
  devices_[channel_num]->getLeftValue();
  devices_[channel_num]->getRightValue();  
}

//----------------------------------------------------------------------
unsigned AlsaMixer::getChannelLeft(unsigned channel_num) const
  throw(InvalidChannelException)
{
  if(!isValidChannel(channel_num))
    throw InvalidChannelException(channel_num);

  return devices_[channel_num]->getLeftValue();
}

//----------------------------------------------------------------------
unsigned AlsaMixer::getChannelRight(unsigned channel_num) const
  throw(InvalidChannelException)
{
  if(!isValidChannel(channel_num))
    throw InvalidChannelException(channel_num);

  return devices_[channel_num]->getRightValue();
}


//----------------------------------------------------------------------
const string& AlsaMixer::getChannelName(unsigned channel_num) const
  throw(InvalidChannelException)
{
  if(!isValidChannel(channel_num))
    throw InvalidChannelException(channel_num);

  return devices_[channel_num]->getLabel();
}

//----------------------------------------------------------------------
bool AlsaMixer::isMuted(unsigned channel_num) const
  throw(InvalidChannelException)
{
  if(!isValidChannel(channel_num))
    throw InvalidChannelException(channel_num);

  return devices_[channel_num]->isMuted();
}

//----------------------------------------------------------------------
bool AlsaMixer::isMono(unsigned channel_num) const
  throw(InvalidChannelException)
{
  if(!isValidChannel(channel_num))
    throw InvalidChannelException(channel_num);

  return !devices_[channel_num]->isStereo();
}

//----------------------------------------------------------------------
bool AlsaMixer::isSupported(unsigned channel_num) const
  throw(InvalidChannelException)
{
  if(!isValidChannel(channel_num))
    throw InvalidChannelException(channel_num);

  return devices_[channel_num]->isSupported();
}

//----------------------------------------------------------------------
bool AlsaMixer::isEnabled(unsigned channel_num) const
  throw(InvalidChannelException)
{
  if(!isValidChannel(channel_num))
    throw InvalidChannelException(channel_num);

  return devices_[channel_num]->isEnabled();
}

//----------------------------------------------------------------------
bool AlsaMixer::canCapture(unsigned channel_num) const
  throw(InvalidChannelException)
{
  if(!isValidChannel(channel_num))
    throw InvalidChannelException(channel_num);

  return devices_[channel_num]->canCapture();
}


//----------------------------------------------------------------------
void AlsaMixer::disableChannel(unsigned channel_num)
  throw(InvalidChannelException)
{
  if(!isValidChannel(channel_num))
    throw InvalidChannelException(channel_num);

  devices_[channel_num]->disable();
}

//----------------------------------------------------------------------
void AlsaMixer::enableChannel(unsigned channel_num)
  throw(InvalidChannelException)
{
  if(!isValidChannel(channel_num))
    throw InvalidChannelException(channel_num);

  devices_[channel_num]->enable();
}





